(*----------------------------------------------------------------------------*
 *  Direct3D sample from DirectX 9.0 SDK Summer 2003                          *
 *  Delphi adaptation by Alexey Barkovoy (e-mail: clootie@ixbt.com)           *
 *                                                                            *
 *  Delphi versions 5-7 are supported                                         *
 *                                                                            *
 *  Modified: 30-Dec-2003                                                     *
 *                                                                            *
 *  Latest version can be downloaded from:                                    *
 *     http://clootie.narod.ru/delphi                                         *
 *----------------------------------------------------------------------------*)
//-----------------------------------------------------------------------------
// File: DolphinVS.cpp
//
// Desc: Sample of swimming dolphin
//
//       Note: This code uses the D3D Framework helper library.
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------

unit DolphinVSUnit;

{$I DirectX.inc}

interface

uses
  Windows, Messages, SysUtils,
  Direct3D9, D3DX9, D3DApp, D3DEnumeration, D3DSettings,
  D3DFile, D3DFont, D3DUtil, DXUtil;

const
  WATER_COLOR   = $00004080;
  D3DVERTEX_FVF = (D3DFVF_XYZ or D3DFVF_NORMAL or D3DFVF_TEX1);

type
  PD3DVertex = ^TD3DVertex;
  TD3DVertex = packed record
    p: TD3DXVector3;
    n: TD3DXVector3;
    tu, tv: Single;
  end;
  PAD3DVERTEX = ^AD3DVERTEX;
  AD3DVERTEX = array[0..0] of TD3DVertex;


  //-----------------------------------------------------------------------------
  // Name: class CMyD3DApplication
  // Desc: Main class to run this application. Most functionality is inherited
  //       from the CD3DApplication base class.
  //-----------------------------------------------------------------------------
  CMyD3DApplication = class(CD3DApplication)
    // Font for drawing text
    m_pFont: CD3DFont;

    // Transform matrices
    m_matWorld: TD3DXMatrixA16;
    m_matView: TD3DXMatrixA16;
    m_matProj: TD3DXMatrixA16;

    // Dolphin object
    m_pDolphinTexture: IDirect3DTexture9;
    m_pDolphinVB1: IDirect3DVertexBuffer9;
    m_pDolphinVB2: IDirect3DVertexBuffer9;
    m_pDolphinVB3: IDirect3DVertexBuffer9;
    m_pDolphinIB: IDirect3DIndexBuffer9;
    m_dwNumDolphinVertices: DWORD;
    m_dwNumDolphinFaces: DWORD;
    m_pDolphinVertexDeclaration: IDirect3DVertexDeclaration9;
    m_pDolphinVertexShader: IDirect3DVertexShader9;
    m_pDolphinVertexShader2: IDirect3DVertexShader9;

    // Seafloor object
    m_pSeaFloorTexture: IDirect3DTexture9;
    m_pSeaFloorVB: IDirect3DVertexBuffer9;
    m_pSeaFloorIB: IDirect3DIndexBuffer9;
    m_dwNumSeaFloorVertices: DWORD;
    m_dwNumSeaFloorFaces: DWORD;
    m_pSeaFloorVertexDeclaration: IDirect3DVertexDeclaration9;
    m_pSeaFloorVertexShader: IDirect3DVertexShader9;
    m_pSeaFloorVertexShader2: IDirect3DVertexShader9;

    // Water caustics
    m_pCausticTextures: array[0..31] of IDirect3DTexture9;
    m_pCurrentCausticTexture: IDirect3DTexture9;

  public
    function OneTimeSceneInit: HRESULT; override;
    function InitDeviceObjects: HRESULT; override;
    function RestoreDeviceObjects: HRESULT; override;
    function InvalidateDeviceObjects: HRESULT; override;
    function DeleteDeviceObjects: HRESULT; override;
    function Render: HRESULT; override;
    function FrameMove: HRESULT; override;
    function FinalCleanup: HRESULT; override;
    function ConfirmDevice(const pCaps: TD3DCaps9; dwBehavior: DWORD;
      adapterFormat: TD3DFormat; backBufferFormat: TD3DFormat): HRESULT; override;

    constructor Create;
  end;

implementation

uses Math;


//-----------------------------------------------------------------------------
// Name: CMyD3DApplication()
// Desc: Constructor
//-----------------------------------------------------------------------------
constructor CMyD3DApplication.Create;
var
  t: DWORD;
begin
  inherited;
  // Override base class members
  m_strWindowTitle         := 'DolphinVS: Tweening Vertex Shader';
  m_d3dEnumeration.AppUsesDepthBuffer := True;

  m_pFont                  := CD3DFont.Create('Arial', 12, D3DFONT_BOLD);

  // Dolphin object
  m_pDolphinTexture        := nil;
  m_pDolphinVB1            := nil;
  m_pDolphinVB2            := nil;
  m_pDolphinVB3            := nil;
  m_pDolphinIB             := nil;
  m_pDolphinVertexDeclaration := nil;
  m_pDolphinVertexShader   := nil;
  m_pDolphinVertexShader2  := nil;

  // SeaFloor object
  m_pSeaFloorTexture       := nil;
  m_pSeaFloorVB            := nil;
  m_pSeaFloorIB            := nil;
  m_pSeaFloorVertexDeclaration := nil;
  m_pSeaFloorVertexShader  := nil;
  m_pSeaFloorVertexShader2 := nil;

  // Water caustics
  for t:= 0 to 31 do m_pCausticTextures[t] := nil;
end;


//-----------------------------------------------------------------------------
// Name: OneTimeSceneInit()
// Desc: Called during initial app startup, this function performs all the
//       permanent initialization.
//-----------------------------------------------------------------------------
function CMyD3DApplication.OneTimeSceneInit:HRESULT;
begin
  Result := S_OK;
end;


//-----------------------------------------------------------------------------
// Name: FrameMove()
// Desc: Called once per frame, the call is the entry point for animating
//       the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FrameMove:HRESULT;
{$WRITEABLECONST ON}
const
  fLight: array[0..3] of Single    = (0.0, 1.0, 0.0, 0.0);
  fLightDolphinSpace: array[0..3] of Single = (0.0, 1.0, 0.0, 0.0);
  fDiffuse: array[0..3] of Single  = (1.00, 1.00, 1.00, 1.00);
  fAmbient: array[0..3] of Single  = (0.25, 0.25, 0.25, 0.25);
  fFog: array[0..3] of Single      = (0.5, 50.0, 1.0/(50.0-1.0), 0.0);
  fCaustics: array[0..3] of Single = (0.05, 0.05, 0, 0);
{$WRITEABLECONST OFF}
var
  fKickFreq, fPhase, fBlendWeight: Single;
  matDolphin, matTrans, matRotate1, matRotate2: TD3DXMatrixA16;
  fWeight1, fWeight2, fWeight3: Single;
  vZero, vOne: TD3DXVector4;
  vWeight: TD3DXVector4;
  tex: DWORD;
  mat, matCamera, matTranspose, matCameraTranspose: TD3DXMatrixA16;
  matViewTranspose, matProjTranspose: TD3DXMatrix;
  matDolphinInv: TD3DXMatrix;
begin
  // Animation attributes for the dolphin
  fKickFreq    := 2*m_fTime;
  fPhase       := m_fTime/3;
  fBlendWeight := Sin(fKickFreq);

  // Move the dolphin in a circle
  D3DXMatrixScaling(matDolphin, 0.01, 0.01, 0.01);
  D3DXMatrixRotationZ(matRotate1, -Cos(fKickFreq)/6);
  D3DXMatrixMultiply(matDolphin, matDolphin, matRotate1);
  D3DXMatrixRotationY(matRotate2, fPhase);
  D3DXMatrixMultiply(matDolphin, matDolphin, matRotate2);
  D3DXMatrixTranslation(matTrans, -5*Sin(fPhase), Sin(fKickFreq)/2, 10-10*Cos(fPhase));
  D3DXMatrixMultiply(matDolphin, matDolphin, matTrans);

  // Animate the caustic textures
  tex := Trunc(m_fTime*32) mod 32;
  m_pCurrentCausticTexture := m_pCausticTextures[tex];

  // Set the vertex shader constants. Note: outside of the blend matrices,
  // most of these values don't change, so don't need to really be set every
  // frame. It's just done here for clarity
  begin
    // Some basic constants
    vZero:= D3DXVector4(0.0, 0.0, 0.0, 0.0);
    vOne:= D3DXVector4(1.0, 0.5, 0.2, 0.05);

    if (fBlendWeight > 0.0) then
    begin
      fWeight1 := abs(fBlendWeight);
      fWeight2 := 1.0 - abs(fBlendWeight);
      fWeight3 := 0.0;
    end else
    begin
      fWeight1 := 0.0;
      fWeight2 := 1.0 - abs(fBlendWeight);
      fWeight3 := abs(fBlendWeight);
    end;
    vWeight:= D3DXVector4(fWeight1, fWeight2, fWeight3, 0.0);

    // Lighting vectors (in world space and in dolphin model space)
    // and other constants
    //  fLight[]    = { 0.0f, 1.0f, 0.0f, 0.0f };
    //  fLightDolphinSpace[] = { 0.0f, 1.0f, 0.0f, 0.0f };
    //  fDiffuse[]  = { 1.00f, 1.00f, 1.00f, 1.00f };
    //  fAmbient[]  = { 0.25f, 0.25f, 0.25f, 0.25f };
    //  fFog[]      = { 0.5f, 50.0f, 1.0f/(50.0f-1.0f), 0.0f };
    //  fCaustics[] = { 0.05f, 0.05f, sinf(m_fTime)/8, cosf(m_fTime)/10 };
    fCaustics[2] := Sin(m_fTime)/8;
    fCaustics[3] := Cos(m_fTime)/10;

    D3DXMatrixInverse(matDolphinInv, nil, matDolphin);
    D3DXVec4Transform(PD3DXVector4(@fLightDolphinSpace)^, PD3DXVector4(@fLight)^, matDolphinInv);
    D3DXVec4Normalize(PD3DXVector4(@fLightDolphinSpace)^, PD3DXVector4(@fLightDolphinSpace)^);

    // Vertex shader operations use transposed matrices
    D3DXMatrixMultiply(matCamera, matDolphin, m_matView);
    D3DXMatrixMultiply(mat, matCamera, m_matProj);
    D3DXMatrixTranspose(matTranspose, mat);
    D3DXMatrixTranspose(matCameraTranspose, matCamera);
    D3DXMatrixTranspose(matViewTranspose, m_matView);
    D3DXMatrixTranspose(matProjTranspose, m_matProj);

    // Set the vertex shader constants
    m_pd3dDevice.SetVertexShaderConstantF( 0, @vZero,     1);
    m_pd3dDevice.SetVertexShaderConstantF( 1, @vOne,      1);
    m_pd3dDevice.SetVertexShaderConstantF( 2, @vWeight,   1);
    m_pd3dDevice.SetVertexShaderConstantF( 4, @matTranspose, 4 );
    m_pd3dDevice.SetVertexShaderConstantF( 8, @matCameraTranspose,  4);
    m_pd3dDevice.SetVertexShaderConstantF(12, @matViewTranspose,  4);
    m_pd3dDevice.SetVertexShaderConstantF(19, @fLightDolphinSpace, 1);
    m_pd3dDevice.SetVertexShaderConstantF(20, @fLight,    1);
    m_pd3dDevice.SetVertexShaderConstantF(21, @fDiffuse,  1);
    m_pd3dDevice.SetVertexShaderConstantF(22, @fAmbient,  1);
    m_pd3dDevice.SetVertexShaderConstantF(23, @fFog,      1);
    m_pd3dDevice.SetVertexShaderConstantF(24, @fCaustics, 1);
    m_pd3dDevice.SetVertexShaderConstantF(28, @matProjTranspose, 4);
  end;

  Result := S_OK;
end;


//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Called once per frame, the call is the entry point for 3d
//       rendering. This function sets up render states, clears the
//       viewport, and renders the scene.
//-----------------------------------------------------------------------------
function CMyD3DApplication.Render: HRESULT;
const
  fAmbientLight: array[0..3] of Single = (0.25, 0.25, 0.25, 0.25);
  fAmbientDark:  array[0..3] of Single = (0.0, 0.0, 0.0, 0.0);
begin
  // Clear the viewport
  m_pd3dDevice.Clear(0, nil, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER,
                     WATER_COLOR, 1.0, 0);

  // Begin the scene
  if SUCCEEDED(m_pd3dDevice.BeginScene) then
  begin
    //FLOAT fAmbientLight[]  = { 0.25f, 0.25f, 0.25f, 0.25f };
    m_pd3dDevice.SetVertexShaderConstantF(22, @fAmbientLight, 1);

    // Render the seafloor
    m_pd3dDevice.SetTexture(0, m_pSeaFloorTexture);
    m_pd3dDevice.SetVertexDeclaration(m_pSeaFloorVertexDeclaration);
    m_pd3dDevice.SetVertexShader(m_pSeaFloorVertexShader);
    m_pd3dDevice.SetStreamSource(0, m_pSeaFloorVB, 0, SizeOf(TD3DVertex));
    m_pd3dDevice.SetIndices(m_pSeaFloorIB);
    m_pd3dDevice.DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0,
                                      0, m_dwNumSeaFloorVertices,
                                      0, m_dwNumSeaFloorFaces);

    // Render the dolphin
    m_pd3dDevice.SetTexture(0, m_pDolphinTexture);
    m_pd3dDevice.SetVertexDeclaration(m_pDolphinVertexDeclaration);
    m_pd3dDevice.SetVertexShader(m_pDolphinVertexShader);
    m_pd3dDevice.SetStreamSource(0, m_pDolphinVB1, 0, SizeOf(TD3DVertex));
    m_pd3dDevice.SetStreamSource(1, m_pDolphinVB2, 0, SizeOf(TD3DVertex));
    m_pd3dDevice.SetStreamSource(2, m_pDolphinVB3, 0, SizeOf(TD3DVertex));
    m_pd3dDevice.SetIndices(m_pDolphinIB);
    m_pd3dDevice.DrawIndexedPrimitive( D3DPT_TRIANGLELIST, 0,
                                      0, m_dwNumDolphinVertices,
                                      0, m_dwNumDolphinFaces);

    // Now, we are going to do a 2nd pass, to alpha-blend in the caustics.
    // The caustics use a 2nd set of texture coords that are generated
    // by the vertex shaders. Lighting from the light above is used, but
    // ambient is turned off to avoid lighting objects from below (for
    // instance, we don't want caustics appearing on the dolphin's
    // underbelly). Finally, fog color is set to black, so that caustics
    // fade in distance.

    // Turn on alpha blending
    m_pd3dDevice.SetRenderState(D3DRS_ALPHABLENDENABLE, 1);

    m_pd3dDevice.SetRenderState(D3DRS_SRCBLEND,  D3DBLEND_ONE);
    m_pd3dDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);

    // Setup the caustic texture
    m_pd3dDevice.SetTexture(0, m_pCurrentCausticTexture);

    // Set ambient and fog colors to black
    // FLOAT fAmbientDark[] = { 0.0f, 0.0f, 0.0f, 0.0f };
    m_pd3dDevice.SetVertexShaderConstantF(22, @fAmbientDark, 1);
    m_pd3dDevice.SetRenderState(D3DRS_FOGCOLOR, $00000000);

    // Render the caustic effects for the seafloor
    m_pd3dDevice.SetVertexDeclaration(m_pSeaFloorVertexDeclaration);
    m_pd3dDevice.SetVertexShader(m_pSeaFloorVertexShader2);
    m_pd3dDevice.SetStreamSource(0, m_pSeaFloorVB, 0, SizeOf(TD3DVertex));
    m_pd3dDevice.SetIndices(m_pSeaFloorIB);
    m_pd3dDevice.DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0,
                                      0, m_dwNumSeaFloorVertices,
                                      0, m_dwNumSeaFloorFaces);

    // Finally, render the caustic effects for the dolphin
    m_pd3dDevice.SetVertexDeclaration(m_pDolphinVertexDeclaration);
    m_pd3dDevice.SetVertexShader(m_pDolphinVertexShader2);
    m_pd3dDevice.SetStreamSource(0, m_pDolphinVB1, 0, SizeOf(TD3DVertex));
    m_pd3dDevice.SetStreamSource(1, m_pDolphinVB2, 0, SizeOf(TD3DVertex));
    m_pd3dDevice.SetStreamSource(2, m_pDolphinVB3, 0, SizeOf(TD3DVertex));
    m_pd3dDevice.SetIndices(m_pDolphinIB);
    m_pd3dDevice.DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0,
                                      0, m_dwNumDolphinVertices,
                                      0, m_dwNumDolphinFaces);

    // Restore modified render states
    m_pd3dDevice.SetRenderState(D3DRS_ALPHABLENDENABLE, 0);
    m_pd3dDevice.SetRenderState(D3DRS_FOGCOLOR, WATER_COLOR);

    // Output statistics
    m_pFont.DrawText(2,  0, D3DCOLOR_ARGB(255,255,255,0), m_strFrameStats);
    m_pFont.DrawText(2, 20, D3DCOLOR_ARGB(255,255,255,0), m_strDeviceStats);

    // End the scene.
    m_pd3dDevice.EndScene;
  end;

  Result := S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Initialize device-dependent objects. This is the place to create mesh
//       and texture objects.
//-----------------------------------------------------------------------------
var
  // Declaration - Create vertex shader for the dolphin
  declDolphin: array[0..9] of TD3DVertexElement9 =
  (
    // First stream is first mesh
    (Stream: 0; Offset: 0;  _Type: D3DDECLTYPE_FLOAT3; Method: D3DDECLMETHOD_DEFAULT; Usage: D3DDECLUSAGE_POSITION; UsageIndex: 0),
    (Stream: 0; Offset: 12; _Type: D3DDECLTYPE_FLOAT3; Method: D3DDECLMETHOD_DEFAULT; Usage: D3DDECLUSAGE_NORMAL;   UsageIndex: 0),
    (Stream: 0; Offset: 24; _Type: D3DDECLTYPE_FLOAT2; Method: D3DDECLMETHOD_DEFAULT; Usage: D3DDECLUSAGE_TEXCOORD; UsageIndex: 0),
    // Second stream is second mesh
    (Stream: 1; Offset: 0;  _Type: D3DDECLTYPE_FLOAT3; Method: D3DDECLMETHOD_DEFAULT; Usage: D3DDECLUSAGE_POSITION; UsageIndex: 1),
    (Stream: 1; Offset: 12; _Type: D3DDECLTYPE_FLOAT3; Method: D3DDECLMETHOD_DEFAULT; Usage: D3DDECLUSAGE_NORMAL;   UsageIndex: 1),
    (Stream: 1; Offset: 24; _Type: D3DDECLTYPE_FLOAT2; Method: D3DDECLMETHOD_DEFAULT; Usage: D3DDECLUSAGE_TEXCOORD; UsageIndex: 1),
    // Third stream is third mesh
    (Stream: 2; Offset: 0;  _Type: D3DDECLTYPE_FLOAT3; Method: D3DDECLMETHOD_DEFAULT; Usage: D3DDECLUSAGE_POSITION; UsageIndex: 2),
    (Stream: 2; Offset: 12; _Type: D3DDECLTYPE_FLOAT3; Method: D3DDECLMETHOD_DEFAULT; Usage: D3DDECLUSAGE_NORMAL;   UsageIndex: 2),
    (Stream: 2; Offset: 24; _Type: D3DDECLTYPE_FLOAT2; Method: D3DDECLMETHOD_DEFAULT; Usage: D3DDECLUSAGE_TEXCOORD; UsageIndex: 2),
    ({D3DDECL_END})
  );
  // Declaration - Create vertex shader for the seafloor
  declSeaFloor: array[0..3] of TD3DVertexElement9 =
  (
    // First stream is first mesh
    (Stream: 0; Offset: 0;  _Type: D3DDECLTYPE_FLOAT3; Method: D3DDECLMETHOD_DEFAULT; Usage: D3DDECLUSAGE_POSITION; UsageIndex: 0),
    (Stream: 0; Offset: 12; _Type: D3DDECLTYPE_FLOAT3; Method: D3DDECLMETHOD_DEFAULT; Usage: D3DDECLUSAGE_NORMAL;   UsageIndex: 0),
    (Stream: 0; Offset: 24; _Type: D3DDECLTYPE_FLOAT2; Method: D3DDECLMETHOD_DEFAULT; Usage: D3DDECLUSAGE_TEXCOORD; UsageIndex: 0),
    ({D3DDECL_END})
  );

function CMyD3DApplication.InitDeviceObjects: HRESULT;
var
  strPath: array[0..MAX_PATH-1] of Char;
  pCode: ID3DXBuffer; // = nil;
  pMeshSourceVB: IDirect3DVertexBuffer9;
  pMeshSourceIB: IDirect3DIndexBuffer9;
  pSrc: PAD3DVERTEX;
  pDst: PAD3DVERTEX;
  DolphinMesh01: CD3DMesh;
  DolphinMesh02: CD3DMesh;
  DolphinMesh03: CD3DMesh;
  SeaFloorMesh: CD3DMesh;
  t, i: DWORD;
  strName: array[0..79] of Char;
begin
  DolphinMesh01:= nil;
  DolphinMesh02:= nil;
  DolphinMesh03:= nil;
  SeaFloorMesh:= nil;
  try
    // Initialize the font's internal textures
    m_pFont.InitDeviceObjects(m_pd3dDevice);

    // Get the path to the texture
    Result:= DXUtil_FindMediaFileCb(strPath, SizeOf(strPath), 'Dolphin.bmp');
    if FAILED(Result) then Exit;
    // Create texture for the dolphin
    if FAILED(D3DXCreateTextureFromFileEx(m_pd3dDevice, strPath,
                D3DX_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT, 0, D3DFMT_UNKNOWN,
                D3DPOOL_MANAGED, D3DX_FILTER_TRIANGLE or D3DX_FILTER_MIRROR,
                D3DX_FILTER_TRIANGLE or D3DX_FILTER_MIRROR, 0, nil, nil, m_pDolphinTexture)) then 
    begin
      Result:= D3DAPPERR_MEDIANOTFOUND;
      Exit;
    end;

    // Get the path to the texture
    Result:= DXUtil_FindMediaFileCb(strPath, SizeOf(strPath), 'SeaFloor.bmp');
    if FAILED(Result) then Exit;
    // Create textures for the seafloor
    if FAILED(D3DXCreateTextureFromFileEx(m_pd3dDevice, strPath,
                D3DX_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT, 0, D3DFMT_UNKNOWN,
                D3DPOOL_MANAGED, D3DX_FILTER_TRIANGLE or D3DX_FILTER_MIRROR,
                D3DX_FILTER_TRIANGLE or D3DX_FILTER_MIRROR, 0, nil, nil, m_pSeaFloorTexture)) then
    begin
      Result:= D3DAPPERR_MEDIANOTFOUND;
      Exit;
    end;

    // Create textures for the water caustics
    Result:= D3DAPPERR_MEDIANOTFOUND;
    for t:= 0 to 31 do
    begin
      StrFmt(strName, 'Caust%.2d.tga', [t]);
      // Get the path to the texture
      if FAILED(DXUtil_FindMediaFileCb(strPath, SizeOf(strPath), strName)) then Exit;
      if FAILED(D3DXCreateTextureFromFileEx( m_pd3dDevice, strPath,
                  D3DX_DEFAULT, D3DX_DEFAULT, D3DX_DEFAULT, 0, D3DFMT_UNKNOWN,
                  D3DPOOL_MANAGED, D3DX_FILTER_TRIANGLE or D3DX_FILTER_MIRROR,
                  D3DX_FILTER_TRIANGLE or D3DX_FILTER_MIRROR, 0, nil, nil, m_pCausticTextures[t]))
      then Exit;
    end;

    DolphinMesh01:= CD3DMesh.Create(nil);
    // Load the file-based mesh objects
    Result:= D3DAPPERR_MEDIANOTFOUND;
    if FAILED(DolphinMesh01.Create_(m_pd3dDevice, PChar('dolphin1.x'))) then Exit;

    DolphinMesh02:= CD3DMesh.Create(nil);
    if FAILED(DolphinMesh02.Create_(m_pd3dDevice, PChar('dolphin2.x'))) then Exit;

    DolphinMesh03:= CD3DMesh.Create(nil);
    if FAILED(DolphinMesh03.Create_(m_pd3dDevice, PChar('dolphin3.x'))) then Exit;

    SeaFloorMesh:= CD3DMesh.Create(nil);
    if FAILED(SeaFloorMesh.Create_(m_pd3dDevice, PChar('SeaFloor.x'))) then Exit;

    // Set the FVF type to match the vertex format we want
    DolphinMesh01.SetFVF(m_pd3dDevice, D3DVERTEX_FVF);
    DolphinMesh02.SetFVF(m_pd3dDevice, D3DVERTEX_FVF);
    DolphinMesh03.SetFVF(m_pd3dDevice, D3DVERTEX_FVF);
    SeaFloorMesh.SetFVF(m_pd3dDevice, D3DVERTEX_FVF);

    // Get the number of vertices and faces for the meshes
    m_dwNumDolphinVertices  := DolphinMesh01.GetSysMemMesh.GetNumVertices;
    m_dwNumDolphinFaces     := DolphinMesh01.GetSysMemMesh.GetNumFaces;
    m_dwNumSeaFloorVertices := SeaFloorMesh.GetSysMemMesh.GetNumVertices;
    m_dwNumSeaFloorFaces    := SeaFloorMesh.GetSysMemMesh.GetNumFaces;

    // Create the dolphin and seafloor vertex and index buffers
    m_pd3dDevice.CreateVertexBuffer(m_dwNumDolphinVertices * SizeOf(TD3DVertex),
                                    D3DUSAGE_WRITEONLY, 0, D3DPOOL_MANAGED,
                                    m_pDolphinVB1, nil);
    m_pd3dDevice.CreateVertexBuffer(m_dwNumDolphinVertices * SizeOf(TD3DVertex),
                                    D3DUSAGE_WRITEONLY, 0, D3DPOOL_MANAGED,
                                    m_pDolphinVB2, nil);
    m_pd3dDevice.CreateVertexBuffer(m_dwNumDolphinVertices * SizeOf(TD3DVertex),
                                    D3DUSAGE_WRITEONLY, 0, D3DPOOL_MANAGED,
                                    m_pDolphinVB3, nil);
    m_pd3dDevice.CreateVertexBuffer(m_dwNumSeaFloorVertices * SizeOf(TD3DVertex),
                                    D3DUSAGE_WRITEONLY, 0, D3DPOOL_MANAGED,
                                    m_pSeaFloorVB, nil);
    m_pd3dDevice.CreateIndexBuffer(m_dwNumDolphinFaces * 3 * sizeof(WORD),
                                   D3DUSAGE_WRITEONLY,
                                   D3DFMT_INDEX16, D3DPOOL_MANAGED,
                                   m_pDolphinIB, nil);
    m_pd3dDevice.CreateIndexBuffer(m_dwNumSeaFloorFaces * 3 * sizeof(WORD),
                                   D3DUSAGE_WRITEONLY,
                                   D3DFMT_INDEX16, D3DPOOL_MANAGED,
                                   m_pSeaFloorIB, nil);

    // Copy vertices for mesh 01
    DolphinMesh01.GetSysMemMesh.GetVertexBuffer(pMeshSourceVB);
    m_pDolphinVB1.Lock(0, 0, Pointer(pDst), 0);
    pMeshSourceVB.Lock(0, 0, Pointer(pSrc), 0);
    Move(pSrc^,pDst^,m_dwNumDolphinVertices * SizeOf(TD3DVertex));
    m_pDolphinVB1.Unlock;
    pMeshSourceVB.Unlock;
    pMeshSourceVB:= nil;

    // Copy vertices for mesh 2
    DolphinMesh02.GetSysMemMesh.GetVertexBuffer(pMeshSourceVB);
    m_pDolphinVB2.Lock(0, 0, Pointer(pDst), 0);
    pMeshSourceVB.Lock(0, 0, Pointer(pSrc), 0);
    Move(pSrc^, pDst^, m_dwNumDolphinVertices * SizeOf(TD3DVertex));
    m_pDolphinVB2.Unlock;
    pMeshSourceVB.Unlock;
    pMeshSourceVB:= nil;

    // Copy vertices for mesh 3
    DolphinMesh03.GetSysMemMesh.GetVertexBuffer( pMeshSourceVB );
    m_pDolphinVB3.Lock(0, 0, Pointer(pDst), 0);
    pMeshSourceVB.Lock(0, 0, Pointer(pSrc), 0);
    move(pSrc^, pDst^, m_dwNumDolphinVertices * SizeOf(TD3DVertex));
    m_pDolphinVB3.Unlock;
    pMeshSourceVB.Unlock;
    pMeshSourceVB:= nil;

    // Copy vertices for the seafloor mesh, and add some bumpiness
    SeaFloorMesh.GetSysMemMesh.GetVertexBuffer(pMeshSourceVB);
    m_pSeaFloorVB.Lock(0, 0, Pointer(pDst), 0);
    pMeshSourceVB.Lock(0, 0, Pointer(pSrc), 0);
    Move(pSrc^, pDst^, m_dwNumSeaFloorVertices * SizeOf(TD3DVertex));
    Randomize; // srand(5);
    for i:= 0 to m_dwNumSeaFloorVertices-1 do
    begin
      pDst[i].p.y := pDst[i].p.y + Random;
      pDst[i].p.y := pDst[i].p.y + Random;
      pDst[i].p.y := pDst[i].p.y + Random;
      pDst[i].tu  := pDst[i].tu * 10;
      pDst[i].tv  := pDst[i].tv * 10;
    end;
    m_pSeaFloorVB.Unlock;
    pMeshSourceVB.Unlock;
    pMeshSourceVB:= nil;

    // Copy indices for the dolphin mesh
    DolphinMesh01.GetSysMemMesh.GetIndexBuffer(pMeshSourceIB);
    m_pDolphinIB.Lock(0, 0, Pointer(pDst), 0);
    pMeshSourceIB.Lock(0, 0, Pointer(pSrc), 0);
    Move(pSrc^, pDst^, 3 * m_dwNumDolphinFaces * SizeOf(Word));
    m_pDolphinIB.Unlock;
    pMeshSourceIB.Unlock;
    pMeshSourceIB:= nil;

    // Copy indices for the seafloor mesh
    SeaFloorMesh.GetSysMemMesh.GetIndexBuffer(pMeshSourceIB);
    m_pSeaFloorIB.Lock(0, 0, Pointer(pDst), 0);
    pMeshSourceIB.Lock(0, 0, Pointer(pSrc), 0);
    Move(pSrc^, pDst^, 3 * m_dwNumSeaFloorFaces * SizeOf(Word));
    m_pSeaFloorIB.Unlock;
    pMeshSourceIB.Unlock;
    pMeshSourceIB:= nil;

    // Create vertex shader for the dolphin
    declDolphin[9]:= D3DDECL_END;
    Result:= m_pd3dDevice.CreateVertexDeclaration(@declDolphin, m_pDolphinVertexDeclaration);
    if FAILED(Result) then Exit;

    Result:= DXUtil_FindMediaFileCb(strPath, SizeOf(strPath), 'DolphinTween.vsh');
    if FAILED(Result) then Exit;
    D3DXAssembleShaderFromFile(strPath, nil, nil, 0, @pCode, nil);
    if FAILED(Result) then Exit;
    Result:= m_pd3dDevice.CreateVertexShader(PDWORD(pCode.GetBufferPointer), m_pDolphinVertexShader);
    SAFE_RELEASE(pCode);
    if FAILED(Result) then Exit;

    Result:= DXUtil_FindMediaFileCb(strPath, sizeof(strPath), 'DolphinTween2.vsh');
    if FAILED(Result) then Exit;
    Result:= D3DXAssembleShaderFromFile(strPath, nil, nil, 0, @pCode, nil);
    if FAILED(Result) then Exit;
    Result:= m_pd3dDevice.CreateVertexShader(PDWORD(pCode.GetBufferPointer), m_pDolphinVertexShader2);
    SAFE_RELEASE(pCode);
    if FAILED(Result) then Exit;

    // Create vertex shader for the seafloor
    declSeaFloor[3]:= D3DDECL_END;
    m_pd3dDevice.CreateVertexDeclaration(@declSeaFloor, m_pSeaFloorVertexDeclaration);
    if FAILED(Result) then Exit;

    Result:= DXUtil_FindMediaFileCb(strPath, sizeof(strPath), 'SeaFloor.vsh');
    if FAILED(Result) then Exit;
    Result:= D3DXAssembleShaderFromFile(strPath, nil, nil, 0, @pCode, nil);
    if FAILED(Result) then Exit;
    Result:= m_pd3dDevice.CreateVertexShader(PDWORD(pCode.GetBufferPointer), m_pSeaFloorVertexShader);
    SAFE_RELEASE(pCode);
    if FAILED(Result) then Exit;

    Result:= DXUtil_FindMediaFileCb(strPath, sizeof(strPath), 'SeaFloor2.vsh');
    if FAILED(Result) then Exit;
    Result:= D3DXAssembleShaderFromFile(strPath, nil, nil, 0, @pCode, nil);
    if FAILED(Result) then Exit;
    Result:= m_pd3dDevice.CreateVertexShader(PDWORD(pCode.GetBufferPointer), m_pSeaFloorVertexShader2);
    SAFE_RELEASE(pCode);
    if FAILED(Result) then Exit;

    Result:= S_OK;
  finally
    DolphinMesh01.Free;
    DolphinMesh02.Free;
    DolphinMesh03.Free;
    SeaFloorMesh.Free;
  end;
end;


//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc: Restore device-memory objects and state after a device is created or
//       resized.
//-----------------------------------------------------------------------------
function CMyD3DApplication.RestoreDeviceObjects:HRESULT;
var
  vEyePt: TD3DXVector3;
  vLookatPt: TD3DXVector3;
  vUpVec: TD3DXVector3;
  fAspect: Single;
begin
  m_pFont.RestoreDeviceObjects;

  // Set the transform matrices
  vEyePt    := D3DXVector3(0.0, 0.0, -5.0);
  vLookatPt := D3DXVector3(0.0, 0.0,  0.0);
  vUpVec    := D3DXVector3(0.0, 1.0,  0.0);
  fAspect := (m_d3dsdBackBuffer.Width) / m_d3dsdBackBuffer.Height;
  D3DXMatrixIdentity(m_matWorld);
  D3DXMatrixLookAtLH(m_matView, vEyePt, vLookatPt, vUpVec);
  D3DXMatrixPerspectiveFovLH(m_matProj, D3DX_PI/3, fAspect, 1.0, 10000.0);

  // Set default render states
  m_pd3dDevice.SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
  m_pd3dDevice.SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
  m_pd3dDevice.SetRenderState(D3DRS_ZENABLE,   1);
  m_pd3dDevice.SetRenderState(D3DRS_FOGENABLE, 1);
  m_pd3dDevice.SetRenderState(D3DRS_FOGCOLOR,  WATER_COLOR);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc: Called when the device-dependent objects are about to be lost.
//-----------------------------------------------------------------------------
function CMyD3DApplication.InvalidateDeviceObjects: HRESULT;
begin
  m_pFont.InvalidateDeviceObjects;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Called when the app is exiting, or the device is being changed,
//       this function deletes any device dependent objects.
//-----------------------------------------------------------------------------
function CMyD3DApplication.DeleteDeviceObjects:HRESULT;
var
  i: DWORD;
begin
  m_pFont.DeleteDeviceObjects;

  // Clean up vertex shaders
  SAFE_RELEASE(m_pDolphinVertexDeclaration);
  SAFE_RELEASE(m_pDolphinVertexShader);
  SAFE_RELEASE(m_pDolphinVertexShader2);
  SAFE_RELEASE(m_pSeaFloorVertexDeclaration);
  SAFE_RELEASE(m_pSeaFloorVertexShader);
  SAFE_RELEASE(m_pSeaFloorVertexShader2);

  // Clean up dolphin objects
  SAFE_RELEASE(m_pDolphinTexture);
  SAFE_RELEASE(m_pDolphinVB1);
  SAFE_RELEASE(m_pDolphinVB2);
  SAFE_RELEASE(m_pDolphinVB3);
  SAFE_RELEASE(m_pDolphinIB);

  // Clean up seafoor objects
  SAFE_RELEASE(m_pSeaFloorTexture);
  SAFE_RELEASE(m_pSeaFloorVB);
  SAFE_RELEASE(m_pSeaFloorIB);

  // Clean up textures for water caustics
  m_pCurrentCausticTexture:= nil;
  for i:= 0 to 31 do SAFE_RELEASE(m_pCausticTextures[i]);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: FinalCleanup()
// Desc: Called before the app exits, this function gives the app the chance
//       to cleanup after itself.
//-----------------------------------------------------------------------------
function CMyD3DApplication.FinalCleanup: HRESULT;
begin
  SAFE_DELETE(m_pFont);
  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: ConfirmDevice()
// Desc: Called during device initialization, this code checks the device
//       for some minimum set of capabilities
//-----------------------------------------------------------------------------
function CMyD3DApplication.ConfirmDevice(const pCaps: TD3DCaps9; dwBehavior: DWORD;
  adapterFormat: TD3DFormat; backBufferFormat: TD3DFormat): HRESULT; 
begin
  Result:= E_FAIL;

  if ((dwBehavior and D3DCREATE_HARDWARE_VERTEXPROCESSING) = D3DCREATE_HARDWARE_VERTEXPROCESSING) or
     ((dwBehavior and D3DCREATE_MIXED_VERTEXPROCESSING) = D3DCREATE_MIXED_VERTEXPROCESSING) then
  begin
    if (pCaps.VertexShaderVersion < D3DVS_VERSION(1,0)) then Exit;
  end;

  // Need to support post-pixel processing (for fog)
  if FAILED(m_pD3D.CheckDeviceFormat(pCaps.AdapterOrdinal, pCaps.DeviceType,
       adapterFormat, D3DUSAGE_RENDERTARGET or D3DUSAGE_QUERY_POSTPIXELSHADER_BLENDING,
       D3DRTYPE_SURFACE, backBufferFormat)) then Exit;

  Result:= S_OK;
end;

end.

